import csv
import time
import numpy as np
import pandas as pd
from datetime import datetime
from optparse import OptionParser

def read_uic_data(data_dir, nrows=None, edges_only=False):
    # Timing
    t1 = time.time();

    # Global dtype dict
    dtype_dict = {'transaction_key': np.int32,
                  'user_from_key': np.int32,
                  'user_key_to': np.int32,
                  'value': np.float32,
                  'public_key_string': str,
                  'transaction_key_string': str}

    # Read public key list
    # Identifier hash associated with nodes
    pub_keys = None if edges_only else pd.io.parsers.read_csv(''.join([data_dir, '/', 'pubkey_list.txt']),
                                                              nrows=nrows,
                                                              names=["public_key_string"], dtype=dtype_dict)

    # Read transaction key list
    # Identifier hash associated with edges
    xn_keys = None if edges_only else pd.io.parsers.read_csv(''.join([data_dir, '/', 'transactionkey_list.txt']),
                                                             nrows=nrows, names=["transaction_key_string"],
                                                             dtype=dtype_dict)

    # Read user Edges
    # Row: Data for a transaction
    user_edges = pd.io.parsers.read_csv(''.join([data_dir, '/', 'user_edges.txt']),
                                        nrows=nrows,
                                        names=["transaction_key", "user_from_key",
                                               "user_key_to", "date", "value"],
                                        dtype=dtype_dict,
                                        index_col=3, parse_dates=[3])
    #user_edges['date'] = [pd.to_datetime(str(d), '%Y%m%d%H%M%S') for d in user_edges['date']]
    #user_edges = user_edges.set_index('date')

    # Read user edge inputs
    # Row: Transaction keys used as input in "this" transaction
    # [tk_self, tk1, tk2, .... , tkn]
    user_edge_inputs = {}
    if not edges_only:
        with open(''.join([data_dir, '/', 'user_edge_inputs.txt']), 'rb') as f:
            reader = csv.reader(f, delimiter=',')
            for idx, ro in enumerate(reader):
                row = np.array(ro, np.int32)
                user_edge_inputs[row[0]] = row[1:]
                if nrows is not None and idx > nrows: break

    # Read input public keys
    # Row: Public keys used for input in "this" transaction
    # [tk_self, pk1, pk2, .... , pkn]
    user_edge_inputs_public_keys = {}
    if not edges_only:
        with open(''.join([data_dir, '/', 'user_edge_input_public_keys.txt']), 'rb') as f:
            reader = csv.reader(f, delimiter=',')
            for idx, ro in enumerate(reader):
                row = np.array(ro, np.int32)
                user_edge_inputs_public_keys[row[0]] = row[1:]
                if nrows is not None and idx > nrows: break

    # Read user key components
    # Row: Public keys belonging to the same user
    # [pk1, pk2, .... , pkn]
    userkey_list = []
    if not edges_only:
        with open(''.join([data_dir, '/', 'userkey_list.txt']), 'rb') as f:
            reader = csv.reader(f, delimiter=',')
            for idx, ro in enumerate(reader):
                userkey_list.append(np.array(ro, np.int32))
                if nrows is not None and idx > nrows: break

    print 'I/O took %3.2f s' % (time.time() - t1)
    return pub_keys, xn_keys, user_edges, user_edge_inputs, user_edge_inputs_public_keys, userkey_list


if __name__ == "__main__":
    parses = OptionParser()
    parses.add_option("-n", "--nrows", dest="nrows",
                      help="Input Number of rows to process",
                      default=100)
    parses.add_option("-d", "--datadir", dest="data_dir",
                      help="Data directory where all the files (pubkey_list.txt etc. ) are located")
    (options, args) = parses.parse_args()

    # Required options =================================================
    if not options.data_dir:
        parses.error('Data Directory not given')

    pub_keys, xn_keys, user_edges, user_edge_inputs, user_edge_inputs_public_keys, userkey_list \
        = read_uic_data(options.data_dir, options.nrows)
    print '\n\n  Public key list ============================================'
    print pub_keys[:10]
    print '\n\n  Transaction key list ======================================='
    print xn_keys[:10]
    print '\n\n  User Edges ================================================='
    print user_edges[:10]
    print '\n\n User edge inputs ==========================================='
    print user_edge_inputs
    print '\n\n Input public keys =========================================='
    print user_edge_inputs_public_keys
    print '\n\n user key components ========================================'
    print userkey_list
